--  speed.ex
--  jiri babor
--  jbabor@paradise.net.nz
--  24 August 2002
                                                     
include machine.e

----------------------------------------------------------------
--  current scheme: {val1,val2,..valn,{key1,key2,..keyn}}
----------------------------------------------------------------

global function index(sequence table, object key)
    return find(key, table[length(table)])
end function

global function fetch(sequence table, object key)
    return table[find(key, table[length(table)])]
end function 

global function set(sequence table, object key, object new_value)
    table[find(key, table[length(table)])] = new_value
    return table
end function 

global function app(sequence table, object key, object val)
    integer n

    n = length(table)
    table = append(table, append(table[n], key))
    table[n] = val
    return  table
end function


----------------------------------------------------------------
--  scheme: {{key1,key2,..keyn},{val1,val2,..valn}}
----------------------------------------------------------------

global function index1(sequence table, object key)
    return find(key, table[1])
end function

global function fetch1(sequence table, object key)
    return table[2][find(key, table[1])]
end function 

global function set1(sequence table, object key, object new_value)
    table[2][find(key, table[1])] = new_value
    return table
end function 

global function app1(sequence table, object key, object val)
    table[1] = append(table[1], key)
    table[2] = append(table[2], val)
    return table
end function

global function store1(sequence table, object key, object val)
    integer i

    i = find(key, table[1])
    if i then
        table[2][i+1] = val
    else
        table[1] = append(table[1], key)
        table[2] = append(table[2], val)
    end if
    return table
end function 


----------------------------------------------------------------
--  scheme: {{key1,key2,..keyn}, val1,val2,..valn}
----------------------------------------------------------------

global function index2(sequence table, object key)
    return find(key, table[1]) + 1
end function

global function fetch2(sequence table, object key)
    return table[find(key, table[1])+1]
end function 

global function set2(sequence table, object key, object new_value)
    table[find(key, table[1])+1] = new_value
    return table
end function 

global function app2(sequence table, object key, object val)
    table[1] = append(table[1], key)
    table = append(table, val)
    return table
end function

global function store2(sequence table, object key, object val)
    integer i

    i = find(key, table[1])             
    if i then
        table[i+1] = val
    else
        table[1] = append(table[1], key)
        table = append(table, val)
    end if
    return table
end function 


----------------------------------------------------------------
--  main
----------------------------------------------------------------

constant n = 10, N = 1000000
object x,z
sequence s,u,v,v1,v2
atom dt,t
integer j

tick_rate(200)

function word(integer len)
    sequence w
    w = {}
    for j = 1 to rand(len) do
        w &= rand(26)+96
    end for
    return w
end function

-- generate random table
s = repeat(0,n)
u = s
for i = 1 to n do
    s[i] = word(6)          -- random key
    u[i] = rand(256)-1      -- random value : byte
end for
v = u & {s}                -- scheme: {val1, ..valN, {key1, ..kN}}
v1 = {s,u}                   -- scheme: {{key1, ..kN}, {val1, ..valN}}
v2 = {s} & u                -- scheme: {{key1, ..kN}, val1, ..valN}

t = time()
for i = 1 to N do
end for
dt = time()-t

----------------------------------------------------------------

puts(1, "comparing schemes: {val1, ..valN, {key1, ..kN}}\n")
puts(1, "                   {{key1, ..kN}, {val1, ..valN}}\n")
puts(1, "                   {{key1, ..kN}, val1, ..valN}\n")
puts(1, "index test...\n")

z = s[length(s)]

t = time()
for i = 1 to N do
    j = index(v, z)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    j = index1(v1, z)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    j = index2(v2, z)
end for
t = time()-t-dt
? t

----------------------------------------------------------------

puts(1, "\nfetch test...\n")

t = time()
for i = 1 to N do
    x = fetch(v, z)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    x = fetch1(v1, z)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    x = fetch2(v2, z)
end for
t = time()-t-dt
? t

----------------------------------------------------------------

z = s[length(s)]

puts(1, "\nset test...\n")

t = time()
for i = 1 to N do
    x = set(v, z, 12345)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    x = set1(v1, z, 12345)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    x = set2(v2, z, 12345)
end for
t = time()-t-dt
? t

----------------------------------------------------------------

puts(1, "\napp test...\n")

t = time()
for i = 1 to N do
    z = app(v,"jiri",12345)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    z = app1(v1,"jiri",12345)
end for
t = time()-t-dt
? t

t = time()
for i = 1 to N do
    z = app2(v2,"jiri",12345)
end for
t = time()-t-dt
? t
