--  tables.e : version 4.11
--  jiri babor
--  jbabor@paradise.net.nz
--  8 September 2002 

----------------------------------------------------------------
--  'single sequence' scheme: {val1, ..valN, {key1, ..kN}}
----------------------------------------------------------------

without warning
without type_check

global constant ET = {{}}           --  empty table

global function index(sequence table, object key)
    -- return *key* index, or zero if key not found
    return find(key, table[length(table)])
end function

global function fetch(sequence table, object key)
    -- return table item with indicated key
    return table[find(key, table[length(table)])]
end function -- fetch

global function set(sequence table, object key, object new_value)
    -- replace specified table item with a new value
    -- return modified table
    table[find(key, table[length(table)])] = new_value
    return table
end function -- set

global function app(sequence table, object key, object val)
    -- add (append) new entry
    integer n

    n = length(table)
    table = append(table, append(table[n], key))
    table[n] = val
    return  table
end function

global function store(sequence table, object key, object val)
    -- store val in table under given key
    -- if matching key is found, corresponding value is replaced
    -- otherwise new key as well as value are appended
    -- return modified table

    integer i,n

    n = length(table)
    i = find(key, table[n])             -- index
    if i then
        table[i] = val
    else
        table = append(table, append(table[n], key))
        table[n] = val
    end if
    return table
end function -- store

global function join(sequence table1, sequence table2)
    --  concatenate table1 and table2
    integer m,n

    m = length(table1)
    n = length(table2)
    return table1[1..m-1] & table2[1..n-1] & {table1[m] & table2[n]} 
end function


global function remove(sequence table, object key)
    --  remove entry with the specified key from the table

    integer i,n

    n = length(table)
    i = find(key, table[n])             -- index
    if i then
        table[n] = table[n][1..i-1] & table[n][i+1..n-1]
        table = table[1..i-1] & table[i+1..n]
    end if
    return table
end function -- remove

global function tsort(sequence s)
    --  shell sort table s by keys in ascending order 

    object t1i,t1j,t2i,t2j              -- temporary objects
    sequence s1,s2
    integer gap,j,first,last,n

    n = length(s)
    s1 = s[n]
    s2 = s[1..n-1]
    last = n-1
    gap = floor(last / 3) + 1
    while 1 do
    	first = gap + 1
    	for i = first to last do
    	    t1i = s1[i]
    	    t2i = s2[i]
    	    j = i - gap
    	    while 1 do
        		t1j = s1[j]
        		t2j = s2[j]
        		if compare(t1i, t1j) >= 0 then
        		    j += gap
        		    exit
        		end if
        		s1[j+gap] = t1j
        		s2[j+gap] = t2j
        		if j <= gap then
        		    exit
        		end if
        		j -= gap
    	    end while
    	    s1[j] = t1i
    	    s2[j] = t2i
    	end for
    	if gap = 1 then
    	    return s2 & {s1}
    	else
    	    gap = floor(gap / 3) + 1
    	end if
    end while
end function -- tsort

global function Fetch(object table, sequence keys)
    --  return item from *nested* table : 'deep' fetch

    for i = 1 to length(keys) do
        table = fetch(table, keys[i])
    end for
    return table
end function -- Fetch

global function Set(sequence table, sequence keys, object new_value)
    --  'deep', recursive set for *nested* tables

    integer i,m,n

    m = length(table)
    n = length(keys)
    i = find(keys[1],table[m])
    if n = 1 then
        table[i] = new_value
    else
        table[i] = Set(table[i], keys[2..n], new_value)
    end if
    return table
end function -- Set

global function App(sequence table, sequence keys, object new_val)
    -- 'deep', recursive add for nested tables
    -- store new_val in table under new (last) key in keys sequence
    -- return modified table

    integer i,m,n

    m = length(table)
    n = length(keys)
    if n = 1 then
        table = append(table, append(table[m], keys[1]))
        table[m] = new_val
    else
        i = find(keys[1], table[m])     -- key index
        table[i] = App(table[i], keys[2..n], new_val)
    end if
    return table
end function

global function dups(sequence table)
    -- return sequence of duplicate keys

    object key
    sequence d,s
    integer n

    d = {}          -- list of duplicates
    n = length(table)
    s = table[n]
    n -= 1
    for i = 1 to n-1 do
        key = s[i]
        if find(key, s[i+1..n]) then
            d = append(d, key)
        end if
    end for
    return d
end function
